/**
 *  scemsite - jan 2016
 *  Author: Cyril TISSOT - Mathias ROUAN (d'après Le modèle AlcoolSurLoup initié à MAPS8 juin 2015 par Lammoglia A., Maestripieri N., Truong Q., Duraffour F., Rouan M.)
 *  Description: Simuler l’évolution des caractéristiques agro-climatiques des parcelles dans les Coteaux du Layon
 */ 
 model Scemsite


global schedules: [world] 
{ /** Insert the global definitions, variables and actions here */

	// définition des couches d'informations	
	file enveloppe_shapefile <- file("../includes/decoup_layon.shp");
	file parcelle_shapefile <- file("../includes/parc_layon.shp");	
	file hydro_shapefile <- file("../includes/cours_eau.shp");	
	file chef_lieu_shapefile <- file("../includes/chef_lieu.shp");
	
	geometry shape <- envelope(enveloppe_shapefile);
	
	int annee <- 2016 update: annee + 1;
	list<int> liste_annee <- list(annee) update : list(annee);
	int horizon_temporel <- 2100;

	int humiditeGlobale;
	int temperatureGlobale;
	int seche <- -1;
	int normale <- 0;
	int humide <- 1;
	int chaude <- 1;
	int froide <- -1;
	
	int scenario <- 1; //normal par défaut
	
	string affichage_scenario;

	float coutMoyenTaille <- 750.0 ; // 75 heures /ha * 10 euros /h (SMIC cout employeur 1500euros)
	float coutMoyenBroyage <- 20.0;
	float coutMoyenFertilisation <- 5.0;
	float coutMoyenDesherbage <- 55.0;
	float coutMoyenEbourgeonnage <- 280.0;
	float coutMoyenPalissage <- 200.0;
	float coutMoyenRognage <- 30.0;
	float coutMoyenEffeuillage <- 150.0;
	float coutMoyenEclaircissage <- 250.0;
	float coutMoyenVendange <- 1350.0;
	float coutMoyenTravailDuSol <- 220.0;
	float coutMoyenTraitementPhytosanitaire <- 50.0;
	float coutMoyenEntretienEnherbement <- 45.0;	
		
	init
	{
		create parcelle from: parcelle_shapefile with: [surface::float(read('surface')),nom_AOC::string(read('nom_AOC'))]
		{
			capital <- gauss(revenu_moyen * 5, revenu_moyen / 5) * surface;
			switch nom_AOC{
									
				match 'Anjou blanc' {revenu_hectare <- 3600.0;}
				match 'Coteaux du Layon' {revenu_hectare <- 7200.0;}
				match 'Superieur' {revenu_hectare <- 9000.0;}
				match 'Premier cru' {revenu_hectare <- 10800.0;}
				match 'Grand cru' {revenu_hectare <- 12000.0;}				
			}			
		}
		
		create hydro from: hydro_shapefile;
		
		create chef_lieu from: chef_lieu_shapefile with: [nom::string(read('NOM'))];
	
	}

	// Descriptif des différentes scenarios climatiques
	
	reflex climatNormal
	{
		if flip(0.4)
		{
			humiditeGlobale <- 0;
			temperatureGlobale <- 0;
		} else
		{
			humiditeGlobale <- rnd(2) - 1;
			temperatureGlobale <- rnd(2) - 1;
		}

	}
		
	reflex dynamiqueClimat
	{

		string climat <- "Année " + annee + " ";
		if temperatureGlobale = -1
		{
			climat <- climat + "Froide";
		} else if temperatureGlobale = 0
		{
			climat <- climat + "Normale";
		} else
		{
			climat <- climat + "Chaude";
		}

		climat <- climat + " et ";
		if humiditeGlobale = -1
		{
			climat <- climat + "Sèche";
		} else if humiditeGlobale = 0
		{
			climat <- climat + "Normale";
		} else
		{
			climat <- climat + "Humide";
		}

		write climat;
	}

	reflex end_simulation when: annee = horizon_temporel
	{
		write ("fin de simulation");
		do pause;
	}

} 
entities {

	// L'agent Parcelle
	species parcelle
	{				
		float surface;				
		int compteur_action <- 15 update: 0 ;
		string nom_AOC;
		float revenu_moyen <- 7200.0; //environ 9000 euros à l'Ha = 35 hectolite /ha * 255 euros / hectolitre		
		float revenuAnnuel;
		float coutAnnuel; // calculé chaque annee
		float coutTaille;
		float coutBroyage;
		float coutFertilisation;
		float coutDesherbage;
		float coutEbourgeonnage;
		float coutPalissage;
		float coutRognage;
		float coutEffeuillage;
		float coutEclaircissage;
		float coutVendange;
		float coutTravailDuSol;
		float coutTraitementPhytosanitaire;
		float coutEntretienEnherbement;
		float fraisFixe;
		float capital;	
		float revenu_hectare; //prix de vente ramené à l'ha		
		
		// Pratiques sur la parcelle		
		action tailler
		{
			coutTaille <- coutMoyenTaille;
			compteur_action<-compteur_action+1;
		}
		action broyer
		{
			coutBroyage <- coutMoyenBroyage;
			compteur_action<-compteur_action+1;
		}	
		action fertiliser
		{
			coutFertilisation <- coutMoyenFertilisation;
			compteur_action<-compteur_action+1;
		}
		action desherber
		{
			switch humiditeGlobale
			{
				match seche	{
					coutDesherbage <- coutMoyenDesherbage * 0;
				}
				match normale {
					coutDesherbage <- coutMoyenDesherbage * 1;
					compteur_action<-compteur_action+1;
				}
				match humide {
					coutDesherbage <- coutMoyenDesherbage * 2;
					compteur_action<-compteur_action+2;
				}
			}
		}
		action ebourgeonner
		{
			coutEbourgeonnage <- coutMoyenEbourgeonnage;
			compteur_action<-compteur_action+1;
		}
		action palisser
		{
			coutPalissage <- coutMoyenPalissage;
			compteur_action<-compteur_action+1;
		}
		action rogner
		{
			if temperatureGlobale = chaude and (humiditeGlobale = humide or humiditeGlobale = normale){
				coutRognage <- coutMoyenRognage * 2;
				compteur_action<-compteur_action+2;
			}
			else{
				coutRognage <- coutMoyenRognage * 1;
				compteur_action<-compteur_action+1;			
			}		
		}
		action effeuiller
		{
			if temperatureGlobale = chaude and humiditeGlobale = humide {
				coutEffeuillage <- coutMoyenEffeuillage * 2;
				compteur_action<-compteur_action+2;
			}
			else if temperatureGlobale = chaude and humiditeGlobale = seche {
				coutEffeuillage <- coutMoyenEffeuillage * 0;			
			}		
			else {
				coutEffeuillage <- coutMoyenEffeuillage * 1;
				compteur_action<-compteur_action+1;
			}
		}
		action eclaircir
		{
				coutEclaircissage <- coutMoyenEclaircissage;
				compteur_action<-compteur_action+1;
		}		
		action vendanger
		{
			coutVendange <- coutMoyenVendange;
			compteur_action<-compteur_action+1;
	
		}	
		action travaillerSol
		{
			switch humiditeGlobale
			{
				match seche
				{
					coutTravailDuSol <- coutMoyenTravailDuSol * 4;
					compteur_action<-compteur_action+4;
				}
	
				match normale
				{
					coutTravailDuSol <- coutMoyenTravailDuSol * 2;
					compteur_action<-compteur_action+2;
				}
	
				match humide
				{
					coutTravailDuSol <- coutMoyenTravailDuSol * 0;
				}	
			}	
		}	
		action traiterPhytosanitaire
		{
			switch humiditeGlobale
			{
				match seche
				{
					coutTraitementPhytosanitaire <- coutMoyenTraitementPhytosanitaire * 2;
					compteur_action<-compteur_action+2;
				}
	
				match normale
				{
					coutTraitementPhytosanitaire <- coutMoyenTraitementPhytosanitaire * 4;
					compteur_action<-compteur_action+4;
				}
	
				match humide
				{
					coutTraitementPhytosanitaire <- coutMoyenTraitementPhytosanitaire * 8;
					compteur_action<-compteur_action+8;
				}	
			}	
		}
		// Calcul des coûts engendrés par les pratiques
		action bilanFinancier
		{
			
			coutAnnuel <- surface * (coutTaille + coutBroyage + coutFertilisation + coutDesherbage + coutEbourgeonnage + coutPalissage + coutRognage + coutEffeuillage + coutEclaircissage + coutVendange + coutTravailDuSol + coutTraitementPhytosanitaire + coutEntretienEnherbement);
			fraisFixe <- coutAnnuel * 0.4;
			revenuAnnuel <- surface * revenu_hectare;
			capital <- capital + (revenuAnnuel - coutAnnuel - fraisFixe);
			
		}
	
		reflex activite
		{		
			do tailler;
			do fertiliser;
			do desherber;
			do ebourgeonner;
			do palisser;
			do rogner;
			do travaillerSol;
			do traiterPhytosanitaire;
			do effeuiller;
			do eclaircir;
			do vendanger;
			do bilanFinancier;
			
		}
		//affichage des parcelles
		aspect draw
		{
			draw shape color: # grey;			
		}			
	}
	
	// Aspects de l'agent hydro
	species hydro
	{
		aspect draw
		{
			draw shape color: # blue;
		}	
	}
	// Aspects de l'agent urbain
	species chef_lieu
	{
		string nom;
		aspect draw
		{
			draw circle(20) color: # black;
			draw nom size: 150 color: # black;
		}	
	}
}

experiment scenario type: gui
{
/** Insert here the definition of the input and output of the model */
		
	parameter "horizon temporel" var: horizon_temporel min: 2050 max: 2100;	
	
	output
	{
		display map type: opengl
		{
			//text affichage_scenario;
			species parcelle aspect: draw;
			species chef_lieu aspect: draw;
			species hydro aspect: draw;
		
		}


		display chart refresh: every(1)
		{
			chart "Nombre moyen d'actions agronomiques" type: series position: { 0, 0 } size: { 1, 0.5 } timexseries:liste_annee y_range: {10 , 25}
			{				
				data "nb d'actions" value: mean(parcelle collect (each.compteur_action)) color: # green;							
			}


			chart "Evolution du rapport prix de vente / coût annuel moyen" type: series position: { 0, 0.5 } size: { 1, 0.5 } timexseries:liste_annee y_range: {0.9 , 1.35}
			{
				data "rapport prix de vente / coût annuel" value: mean(parcelle collect (each.coutAnnuel)) > 0 ? mean(parcelle collect (each.revenuAnnuel))/(mean(parcelle collect (each.coutAnnuel)) + mean(parcelle collect (each.fraisFixe))) : 1  color: # blue;	
			}
		}
	}
}




