/**
 *  scemsite - jan 2016
 *  Author: Cyril TISSOT - Mathias ROUAN (d'après Le modèle AlcoolSurLoup initié à MAPS8 juin 2015 par Lammoglia A., Maestripieri N., Truong Q., Duraffour F., Rouan M.)
 *  Description: Simuler l’évolution des caractéristiques agro-climatiques des parcelles dans les Coteaux du Layon
 */ 
 model Scemsite


global schedules: [world] 
{ /** Insert the global definitions, variables and actions here */


	map profils_annees_climatiques <- [1990::[2126, "chaude"], 1991::[1513, "froide"], 1992::[1728, "froide"], 
		1993::[1512, "froide"], 1994::[1993, "chaude"], 1995::[1979, "chaude"], 1996::[1604, "froide"], 
		1997::[2018, "chaude"], 1998::[1701, "normale"], 1999::[1990, "chaude"], 2000::[1854, "normale"],
		2001::[1771, "froide"], 2002::[1933, "chaude"], 2003::[2072, "chaude"], 2004::[1814, "normale"],
		2005::[1936, "chaude"], 2006::[1998, "chaude"], 2007::[1835, "normale"], 2008::[1717, "froide"],
		2009::[1937, "chaude"], 2010::[1568, "froide"], 2011::[2276, "chaude"], 2012::[1728, "froide"],
		2013::[1693, "froide"]
	];

	
	// définition des couches d'informations	
	file enveloppe_shapefile <- file("../includes/decoup_layon.shp");
	file parcelle_shapefile <- file("../includes/parc_layon.shp");
	file capteur_shapefile <- file("../includes/capteurs.shp");
	file hydro_shapefile <- file("../includes/cours_eau.shp");	
	file chef_lieu_shapefile <- file("../includes/chef_lieu.shp");
	
	geometry shape <- envelope(enveloppe_shapefile);
	
	int annee <- 2016 update: annee + 1;
	list<int> liste_annee <- list(annee) update : list(annee);
	int horizon_temporel <- 2100;
	
	int seuil_temperature_AOC <- 1900;

	int humiditeGlobale;
	int temperatureGlobale;
	int seche <- -1;
	int normale <- 0;
	int humide <- 1;
	int chaude <- 1;
	int froide <- -1;
	int scenario <- 1; //normal par défaut
	
	string affichage_scenario;

	
	float coutMoyenTaille <- 750.0 ; // 75 heures /ha * 10 euros /h (SMIC cout employeur 1500euros)
	float coutMoyenBroyage <- 20.0;
	float coutMoyenFertilisation <- 5.0;
	float coutMoyenDesherbage <- 55.0;
	float coutMoyenEbourgeonnage <- 280.0;
	float coutMoyenPalissage <- 200.0;
	float coutMoyenRognage <- 30.0;
	float coutMoyenEffeuillage <- 150.0;
	float coutMoyenEclaircissage <- 250.0;
	float coutMoyenVendange <- 1350.0;
	float coutMoyenTravailDuSol <- 220.0;
	float coutMoyenTraitementPhytosanitaire <- 50.0;
	float coutMoyenEntretienEnherbement <- 45.0;	
	
	
	init
	{
		
		create capteur from: capteur_shapefile with: [indice::string(read('Indice')), code::int(read('code'))];

		create parcelle from: parcelle_shapefile with: [surface::float(read('surface')),nom_AOC::string(read('nom_AOC'))]
		{
			capteur_parcelle <- capteur closest_to (self);
			color_capteur <- hsb(capteur_parcelle.code/25, capteur_parcelle.code/25, capteur_parcelle.code/25);			
			switch nom_AOC{
									
				match 'Anjou blanc' {color_AOC <- # lightgrey;}
				match 'Coteaux du Layon' {color_AOC <- # lightgreen;}
				match 'Superieur' {color_AOC <- # yellow;}
				match 'Premier cru' {color_AOC <- # orange;}
				match 'Grand cru' {color_AOC <- # red;}				
			}		
			capital <- gauss(revenu_moyen * 5, revenu_moyen / 5) * surface;
			
		}
		
		

					
		create hydro from: hydro_shapefile;
		
		create chef_lieu from: chef_lieu_shapefile with: [nom::string(read('NOM'))];
		
		create sauvegarde;
				
	}


	// Descriptif des différentes scenarios climatiques
	action climatFroidHumide
	{
		if flip(0.4)
		{
			humiditeGlobale <- 1;
			temperatureGlobale <- -1;
		} else
		{
			humiditeGlobale <- rnd(2) - 1;
			temperatureGlobale <- rnd(2) - 1;
		}

	}
	action climatNormal
	{
		if flip(0.4)
		{
			humiditeGlobale <- 0;
			temperatureGlobale <- 0;
		} else
		{
			humiditeGlobale <- rnd(2) - 1;
			temperatureGlobale <- rnd(2) - 1;
		}

	}

	action climatChaudSec
	{
		if flip(0.4)
		{
			humiditeGlobale <- -1;
			temperatureGlobale <- 1;
		} else
		{
			humiditeGlobale <- rnd(2) - 1;
			temperatureGlobale <- rnd(2) - 1;
		}

	}

	action climatTresChaud
	{
		if flip(0.6)
		{
			humiditeGlobale <- rnd(2) - 1;
			temperatureGlobale <- 1;
		} else
		{
			humiditeGlobale <- rnd(2) - 1;
			temperatureGlobale <- rnd(2) - 1;
		}

	}

	reflex dynamiqueClimat
	{
		switch scenario
		{
			match 0 {
				do climatFroidHumide();
				affichage_scenario <- "Scénario humide et froid";
			}
			match 1	{
				do climatNormal();
				affichage_scenario <- "Scénario normal";
			}
			match 2	{
				do climatChaudSec();
				affichage_scenario <- "Scénario chaud et sec";
			}
			match 3	{
				do climatTresChaud();
				affichage_scenario <- "Scénario très chaud";
			}			

		}


		string climat <- "Année " + annee + " ";
		if temperatureGlobale = -1
		{
			climat <- climat + "Froide";
		} else if temperatureGlobale = 0
		{
			climat <- climat + "Normale";
		} else
		{
			climat <- climat + "Chaude";
		}

		climat <- climat + " et ";
		if humiditeGlobale = -1
		{
			climat <- climat + "Sèche";
		} else if humiditeGlobale = 0
		{
			climat <- climat + "Normale";
		} else
		{
			climat <- climat + "Humide";
		}

		write climat;
	}

	reflex end_simulation when: annee = horizon_temporel
	{
		write ("fin de simulation");
		do pause;
	}

} 
entities {
	// L'agent capteur

	species capteur
	{
		int code;
		string indice; //caractéristique du capteur (de 1 à 4 en fonction de sa localisation, altitude, emplacement, exposition....
		int region_ID;
		int tempe_station;
		reflex mesurer_climat
		{
		// définition des valeurs standards
			switch temperatureGlobale
			{
				match froide
				{
					tempe_station <- int(one_of(profils_annees_climatiques where (each contains "froide")) at 0); //2000;
					//tempe_station <- 1500;
				}
	
				match normale
				{
					tempe_station <- int((one_of(profils_annees_climatiques where (each contains "normale"))) at 0); //2000;
					//tempe_station <- 1900;
				}
	
				match chaude
				{
					tempe_station <- int((one_of(profils_annees_climatiques where (each contains "chaude"))) at 0); //2000;
					//tempe_station <- 2100;
				}
	
			}
	
			//variabilité en fonction de la localisation de la station
			switch indice
			{
				match "1"
				{
					tempe_station <- tempe_station + (rnd(27) - 150);
				}
	
				match "2"
				{
					tempe_station <- tempe_station + (rnd(31) - 122);
				}
	
				match "3"
				{
					tempe_station <- tempe_station + (rnd(35) - 91);
				}
				match "4"
				{
					tempe_station <- tempe_station + (rnd(207) - 55);
				}
			}
	
		}
		//affichage des stations
		aspect draw
		{
			switch indice
			{
				match "1"
				{
					draw circle(50 # m) color: # blue border: # white;
				}
	
				match "2"
				{
					draw circle(50 # m) color: # green border: # white;
				}
	
				match "3"
				{
					draw circle(50 # m) color: # orange border: # white;
				}
				match "4"
				{
					draw circle(50 # m) color: # red border: # white;
				}
	
			}
	
		}
		aspect draw_capteur
		{
			draw circle(50 # m) color: hsb(code/25, code/25, code/25) border: # white;		
		}

	
	}
	

	
	
	// L'agent Parcelle
	species parcelle
	{
		var color_capteur type: rgb ;
		var color_AOC type: rgb ;
		capteur capteur_parcelle; // région de climat
		float surface;
		string aoc;		
		int compteur_action <- 15 update: 0 ;
		string nom_AOC;
		
		
		float revenu_moyen <- 7200.0; //environ 9000 euros à l'Ha = 35 hectolite /ha * 255 euros / hectolitre
			
		float revenuAnnuel;
		float coutAnnuel; // calculé chaque annee
		float coutTaille;
		float coutBroyage;
		float coutFertilisation;
		float coutDesherbage;
		float coutEbourgeonnage;
		float coutPalissage;
		float coutRognage;
		float coutEffeuillage;
		float coutEclaircissage;
		float coutVendange;
		float coutTravailDuSol;
		float coutTraitementPhytosanitaire;
		float coutEntretienEnherbement;
		float fraisFixe;
			
		float capital;
	
		float revenu_hectare; //prix de vente ramené à l'ha
		
		// Pratiques sur la parcelle		
		action tailler
		{
			coutTaille <- coutMoyenTaille;
			compteur_action<-compteur_action+1;
		}
		action broyer
		{
			coutBroyage <- coutMoyenBroyage;
			compteur_action<-compteur_action+1;
		}	
		action fertiliser
		{
			coutFertilisation <- coutMoyenFertilisation;
			compteur_action<-compteur_action+1;
		}
		action desherber
		{
			switch humiditeGlobale
			{
				match seche	{
					coutDesherbage <- coutMoyenDesherbage * 0;
				}
				match normale {
					coutDesherbage <- coutMoyenDesherbage * 1;
					compteur_action<-compteur_action+1;
				}
				match humide {
					coutDesherbage <- coutMoyenDesherbage * 2;
					compteur_action<-compteur_action+2;
				}
			}
		}
		action ebourgeonner
		{
			coutEbourgeonnage <- coutMoyenEbourgeonnage;
			compteur_action<-compteur_action+1;
		}
		action palisser
		{
			coutPalissage <- coutMoyenPalissage;
			compteur_action<-compteur_action+1;
		}
		action rogner
		{
			if temperatureGlobale = chaude and (humiditeGlobale = humide or humiditeGlobale = normale){
				coutRognage <- coutMoyenRognage * 2;
				compteur_action<-compteur_action+2;
			}
			else{
				coutRognage <- coutMoyenRognage * 1;
				compteur_action<-compteur_action+1;			
			}		
		}
		action effeuiller
		{
			if temperatureGlobale = chaude and humiditeGlobale = humide {
				coutEffeuillage <- coutMoyenEffeuillage * 2;
				compteur_action<-compteur_action+2;
			}
			else if temperatureGlobale = chaude and humiditeGlobale = seche {
				coutEffeuillage <- coutMoyenEffeuillage * 0;			
			}		
			else {
				coutEffeuillage <- coutMoyenEffeuillage * 1;
				compteur_action<-compteur_action+1;
			}
		}
		action eclaircir
		{
				coutEclaircissage <- coutMoyenEclaircissage;
				compteur_action<-compteur_action+1;
		}
		
		action vendanger
		{
			coutVendange <- coutMoyenVendange;
			compteur_action<-compteur_action+1;
	
		}
	
		action travaillerSol
		{
			switch humiditeGlobale
			{
				match seche
				{
					coutTravailDuSol <- coutMoyenTravailDuSol * 4;
					compteur_action<-compteur_action+4;
				}
	
				match normale
				{
					coutTravailDuSol <- coutMoyenTravailDuSol * 2;
					compteur_action<-compteur_action+2;
				}
	
				match humide
				{
					coutTravailDuSol <- coutMoyenTravailDuSol * 0;
				}
	
			}
	
		}
	
		action traiterPhytosanitaire
		{
			switch humiditeGlobale
			{
				match seche
				{
					coutTraitementPhytosanitaire <- coutMoyenTraitementPhytosanitaire * 2;
					compteur_action<-compteur_action+2;
				}
	
				match normale
				{
					coutTraitementPhytosanitaire <- coutMoyenTraitementPhytosanitaire * 4;
					compteur_action<-compteur_action+4;
				}
	
				match humide
				{
					coutTraitementPhytosanitaire <- coutMoyenTraitementPhytosanitaire * 8;
					compteur_action<-compteur_action+8;
				}
	
			}
	
		}
		// Calcul des coûts engendrés par les pratiques
		action bilanFinancier
		{
			
			coutAnnuel <- surface * (coutTaille + coutBroyage + coutFertilisation + coutDesherbage + coutEbourgeonnage + coutPalissage + coutRognage + coutEffeuillage + coutEclaircissage + coutVendange + coutTravailDuSol + coutTraitementPhytosanitaire + coutEntretienEnherbement);
			fraisFixe <- coutAnnuel * 0.4;
			revenuAnnuel <- surface * revenu_hectare;
			capital <- capital + (revenuAnnuel - coutAnnuel - fraisFixe);
			
		}
	
		
	
		reflex activite
		{		
			do tailler;
			do fertiliser;
			do desherber;
			do ebourgeonner;
			do palisser;
			do rogner;
			do travaillerSol;
			do traiterPhytosanitaire;
			do effeuiller;
			do eclaircir;
			do vendanger;
			do bilanFinancier;
			
		}
		//affichage des parcelles
		aspect draw
		{
			draw shape color: capteur_parcelle.tempe_station > seuil_temperature_AOC  ? # green : color_AOC;
			
		}
		//affichage en fonctions des code_capteur
		aspect draw_capteur
		{
			
			draw shape color: color_capteur;
			
		}
	
	}
	
	
		
	
	// Aspects de l'agent hydro
	species hydro
	{
		aspect draw
		{
			draw shape color: # blue;
		}
	
	}
	// Aspects de l'agent urbain
	species chef_lieu
	{
		string nom;
		aspect draw
		{
			draw circle(20) color: # black;
			draw nom size: 150 color: # black;
		}
	
	}
	species sauvegarde 
	{
		reflex  sauvegarde_csv{
			if cycle = 0
			{
				save "annee,3600,9000,10800" to : "result_scen1_revenu.csv" type: csv;
			}
	
			//save "" + annee + "," + length(parcelle where (each.revenu_hectare = each.revenu_moyen_AOC * 0.4)) + "," + length(parcelle where (each.revenu_hectare = revenu_moyen_AOC))+ "," + length(parcelle where (each.revenu_hectare = revenu_moyen_AOC * 1.2)) to : "result_scen1_revenu.csv" type: csv; //
		
		}
	
	
	}

}

experiment scenario type: gui
{
/** Insert here the definition of the input and output of the model */
	parameter "Scénario climatique" var: scenario min: -1 max: 4;
	
	parameter "horizon temporel" var: horizon_temporel min: 2050 max: 2100;
	
	output
	{
		display map type: opengl
		{
		//text affichage_scenario;

			//species parcelle aspect: draw_capteur;
			species parcelle aspect: draw;
			species chef_lieu aspect: draw;
			species hydro aspect: draw;
			species capteur aspect: draw;			
		}

		display chart refresh: every(1)
		{
			chart "Nombre moyen d'actions agronomiques" type: series position: { 0, 0 } size: { 1, 0.5 } timexseries:liste_annee y_range: {10 , 25}
			{				
				data "nb d'actions" value: mean(parcelle collect (each.compteur_action)) color: # green;							
			}


			chart "Evolution du rapport prix de vente / coût annuel moyen" type: series position: { 0, 0.5 } size: { 1, 0.5 } timexseries:liste_annee //y_range: {0.9 ,1.75}
			{
				data "Anjou blanc" value: mean(parcelle where (each.nom_AOC = 'Anjou blanc') collect (each.coutAnnuel)) > 0 ? mean(parcelle where (each.nom_AOC = 'Anjou blanc') collect (each.revenuAnnuel))/(mean(parcelle where (each.nom_AOC = 'Anjou blanc') collect (each.coutAnnuel)) + mean(parcelle where (each.nom_AOC = 'Anjou blanc') collect (each.fraisFixe))) : 1  color: # lightgrey;
				data "Coteaux du Layon" value: mean(parcelle where (each.nom_AOC = 'Coteaux du Layon') collect (each.coutAnnuel)) > 0 ? mean(parcelle where (each.nom_AOC = 'Coteaux du Layon') collect (each.revenuAnnuel))/(mean(parcelle where (each.nom_AOC = 'Coteaux du Layon') collect (each.coutAnnuel)) + mean(parcelle where (each.nom_AOC = 'Coteaux du Layon') collect (each.fraisFixe))) : 1  color: # lightgreen;
				data "Superieur" value: mean(parcelle where (each.nom_AOC = 'Superieur') collect (each.coutAnnuel)) > 0 ? mean(parcelle where (each.nom_AOC = 'Superieur') collect (each.revenuAnnuel))/(mean(parcelle where (each.nom_AOC = 'Superieur') collect (each.coutAnnuel)) + mean(parcelle where (each.nom_AOC = 'Superieur') collect (each.fraisFixe))) : 1  color: # yellow;
				data "Premier cru" value: mean(parcelle where (each.nom_AOC = 'Premier cru') collect (each.coutAnnuel)) > 0 ? mean(parcelle where (each.nom_AOC = 'Premier cru') collect (each.revenuAnnuel))/(mean(parcelle where (each.nom_AOC = 'Premier cru') collect (each.coutAnnuel)) + mean(parcelle where (each.nom_AOC = 'Premier cru') collect (each.fraisFixe))) : 1  color: # orange;
				data "Grand cru" value: mean(parcelle where (each.nom_AOC = 'Grand cru') collect (each.coutAnnuel)) > 0 ? mean(parcelle where (each.nom_AOC = 'Grand cru') collect (each.revenuAnnuel))/(mean(parcelle where (each.nom_AOC = 'Grand cru') collect (each.coutAnnuel)) + mean(parcelle where (each.nom_AOC = 'Grand cru') collect (each.fraisFixe))) : 1  color: # red;
	
			}
		}

	}

}

experiment influence_capteurs type: gui
{
/** Insert here the definition of the input and output of the model */
	
	output
	{
		display map type: opengl
		{
		//text affichage_scenario;

			species parcelle aspect: draw_capteur;
			
			species chef_lieu aspect: draw;
			species hydro aspect: draw;
			species capteur aspect: draw_capteur;			
		}


	}

}



